/****************************************************************************
*
*	  Module Filename:  Molecule_Type.cpp
*	  Module Name:      Molecule_Type
*	  Description:
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

//	Globally used defines

#include  "globldef.h"
#include  "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

//--------------  Chemistry Subsystem  --------------------------------------------------------------------------
#include  "Molecule_Type.h"
#include  "Chemistry.h"
//---------------------------------------------------------------------------------------------------------------

//--------------  Reactor Subsystem  --------------------------------------------------------------------------
//#include  "Realised_Molecule_Type.h"
//---------------------------------------------------------------------------------------------------------------


#include  <gsl/gsl_rng.h>    // GNU Scientific library
//---------------- Utilities Subsystem ----------------------------------------------------------------------------
#include  "Random_Sequence.h"
//----------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------

//  Initialise Static Data

    int Molecule_Type::Next_Molecule_Type_Identifier = 0;
    int Molecule_Type::Max_Molecule_Mass = 5;
    double Molecule_Type::Max_MolType_Pot_Energy = 5;

//------------------------------------------------------------------------------
void  Molecule_Type::Add_To_Chemistry()

//  Add the Molecule_Type to the Chemistry

{
    Chemistry::Get_Chemistry__Ptr()->Add_Mol_Type(this);
}
//-----------------------------------------------------------------------------------------------------------------

void  Molecule_Type::Clear_Lists_Of_Interaction_Types()

//  Clear the lists of Interaction Types with this Molecule Type as Input and Product

{
    List_Of_Int_Types_With_MolType_As_Input.Clear();
    List_Of_Int_Types_With_MolType_As_Product.Clear();
}

//------------------------------------------------------------------------------
/*
void Molecule_Type::Draw(Network_Painter_Prot* const Network_Painter_Prot__Ptr,int Number_Of_Molecules)

//  Draw the Molecule_Type on the specified Network_Painter.

{
Network_Painter_Prot__Ptr->Draw_Molecule_Type(Identifier,
                                         Number_Of_Molecules,
                                         EOpt_Node_Not_Selected,
                                         true); // Draw for first time, so create a Network Display Item
                                                // in case need to redraw;
}*/
//----------------------------------------------------------------------------------------------------------
int     Molecule_Type::Get_Identifier()  const

//  Get the Identifier for the Molecule_Type

{
    return(Identifier);
}
//----------------------------------------------------------------------------------------------------------

int     Molecule_Type::Get_Mass()  const

//  Get the Mass for the Molecule_Type

{
    return(Mass);
}
//---------------------------------------------------------------------------------------------------------

double     Molecule_Type::Get_Potential_Energy()  const

//  Get the Potential for the Molecule_Type

{
    return(Potential_Energy);
}

//----------------------------------------------------------------------------------------------------------
Realised_Molecule_Type*     Molecule_Type::Get_Realised_Molecule_Type__Ptr() const

//  Get pointer to Realised Molecule Type associated with this Molecule_Type

{

    return(Realised_Molecule_Type__Ptr);
}

//------------------------------------------------------------------------------
int    Molecule_Type::Get_Specification()

//  Get the Specification for the Molecule_Type

{

    return(Specification);
}
//------------------------------------------------------------------------------

void Molecule_Type::Initialise(int MolType_Mass, double MolType_Pot_Energy)

//  Standard Initialisation for Molecule_Type

{
    TRACE("Molecule_Type::Initialise");
    Identifier = Next_Molecule_Type_Identifier;
    Specification = Identifier; //  Same as Identifier for Random Chemistry
    Mass = MolType_Mass;
    Potential_Energy = MolType_Pot_Energy;
    Next_Molecule_Type_Identifier++;

//  Initially the Molecule_Type is not linked to any Realised_Molecule_Type
//  in the Reactor

    Realised_Molecule_Type__Ptr = NULL;
}
//-------------------------------------------------------------------------------------------------------------

//bool  Molecule_Type::Is_In_Reactor()

//  Check whether the Molecute_Type currently exists in the Reactor.
//  If so return true, otherwise return false

//{
//    if (Realised_Molecule_Type__Ptr == NULL)
//    THEN
//        return(false);
 //   ELSEIF (Realised_Molecule_Type__Ptr->Get_Num_Of_Mols_In_Reactor() == 0)
 //   THEN
 //       return (false);
 //   ELSE
//        return(true);     
 //   ENDIF
//}
//----------------------------------------------------------------------------------------------------------


void    Molecule_Type::Link_To_Realised_Molecule_Type
                        (Realised_Molecule_Type*  Realised_Mol_Type_Arg__Ptr)

//  Setup a pointer in the Molecule_Type to the Realised_Molecule_Type
//  in the Reactor

{
    Realised_Molecule_Type__Ptr = Realised_Mol_Type_Arg__Ptr;
}
//---------------------------------------------------------------------------------------------------------------

Molecule_Type::Molecule_Type()

//  Constructor for cases where no Molecule Type is specified
//  Choose a random Mass

{   
    TRACE("Molecule_Type::Molecule_Type()");
    int MolType_Mass  = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_Molecule_Mass) + 1;  // Avoid Mass = 0
    double MolType_Pot_Energy  = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_MolType_Pot_Energy); 
    Initialise(MolType_Mass, MolType_Pot_Energy);
}
//----------------------------------------------------------------------------------------------------------------

Molecule_Type::Molecule_Type(Mass_Tp  MolType_Mass)

//  Constructor for cases where Mass is known
//  Set the Potential Energy at random

{
    TRACE("Molecule_Type::Molecule_Type(Mass_Tp  MolType_Mass");
    double MolType_Pot_Energy  = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_MolType_Pot_Energy); 
    Initialise(MolType_Mass.Get_Value(), MolType_Pot_Energy);
}
//----------------------------------------------------------------------------------------------------------------

Molecule_Type::Molecule_Type(Potential_Energy_Tp  MolType_Pot_Energy)

//  Constructor for cases where Potential Energy is known

{
    TRACE("Molecule_Type::Molecule_Type(Potential_Energy_Tp  MolType_Pot_Energy)");
    int MolType_Mass  = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_Molecule_Mass) + 1;  // Avoid Mass = 0
    Initialise(MolType_Mass, MolType_Pot_Energy.Get_Value());
}
//---------------------------------------------------------------------------------------------------------------

Molecule_Type::Molecule_Type(Mass_Tp  MolType_Mass,  Potential_Energy_Tp  MolType_Pot_Energy)

//  Constructor for case where  Mass and Potential Energy are known

{    
    TRACE("Molecule_Type::Molecule_Type(Mass_Tp  MolType_Mass,  Potential_Energy_Tp  MolType_Pot_Energy)");
    Initialise(MolType_Mass.Get_Value(), MolType_Pot_Energy.Get_Value());
}
//----------------------------------------------------------------------------------------------------------

void    Molecule_Type::Set_Identifier(int   Identifier_Value)

//  Get the Identifier for the Molecule_Type

{
    Identifier = Identifier_Value;
}
//----------------------------------------------------------------------------------------------------------

void    Molecule_Type::Set_Specification(int Specification_Value)

//  Set the Specification for the Molecule_Type

{
    Specification = Specification_Value;
}
//----------------------------------------------------------------------------------------------------------------

void  Molecule_Type::Unlink_From_R_Mol_Type()

//  Remove the link from the Molecule Type to a Realised Molecule Type

{
    Realised_Molecule_Type__Ptr = 0;
}
//----------------------------------------------------------------------------------------------------------
void    Molecule_Type::List_Of_Int_Types_With_MolType_As_Product_Class::Add(Interaction_Type*  Int_Type__Ptr)

//  Add the specified Interaction_Type to the list of Interaction_Types that have
//  this Molecule_Type as a Product

{
    Vector__Int_Types_For_Product.push_back(Int_Type__Ptr);
}
//----------------------------------------------------------------------------------------------------------------
void    Molecule_Type::List_Of_Int_Types_With_MolType_As_Product_Class::Clear()

//   Clear the Molecule Type's List of Interaction Types with the Mol type as a Product

{
    Vector__Int_Types_For_Product.clear();
}
//----------------------------------------------------------------------------------------------------------
Interaction_Type*  Molecule_Type::List_Of_Int_Types_With_MolType_As_Product_Class::Get_Current_Item()

//  Return the current item in the list 



{
    return *Iter_Int_Type_With_Mol_Type_As_Product;
}
//----------------------------------------------------------------------------------------------------------

bool    Molecule_Type::List_Of_Int_Types_With_MolType_As_Product_Class::Iter_To_First()

//  Move the list Iterator to the first item in the list

{
    if (Vector__Int_Types_For_Product.empty() == true)
    THEN
        return false;
    ELSE
        Iter_Int_Type_With_Mol_Type_As_Product = Vector__Int_Types_For_Product.begin();
        return true;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
bool     Molecule_Type::List_Of_Int_Types_With_MolType_As_Product_Class::Iter_To_Next()

//  Move the List Iterator to the next item in the list

{

    Assert(Iter_Int_Type_With_Mol_Type_As_Product != Vector__Int_Types_For_Product.end(),
           "Molecule_Type::List_Of_Int_Types_With_MolType_As_Product_Class::Iter_To_Next - Invalid Argument");


    Iter_Int_Type_With_Mol_Type_As_Product++;

    if (Iter_Int_Type_With_Mol_Type_As_Product == Vector__Int_Types_For_Product.end())
    THEN
        return false;
    ELSE
        return true;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void    Molecule_Type::List_Of_Int_Types_With_MolType_As_Product_Class::Remove(Interaction_Type* Int_Type__Ptr)

//  Remove the specified Interaction_Type from the Molecule_type's list of products
//  This function is not optimised, and so is suitable only for use when infrequently called

{
    vector<Interaction_Type*>::iterator  Iter;
    Iter = find(Vector__Int_Types_For_Product.begin(), Vector__Int_Types_For_Product.end(), Int_Type__Ptr);

    Assert (Iter != Vector__Int_Types_For_Product.end(),
            "Molecule_Type::List_Of_Int_Types_With_MolType_As_Product_Class::Remove - Invalid argument");
    Vector__Int_Types_For_Product.erase(Iter);        
}

//----------------------------------------------------------------------------------------------------------
void    Molecule_Type::List_Of_Int_Types_With_MolType_As_Input_Class::Add(Interaction_Type*  Int_Type__Ptr)

//  Add the specified Interaction_Type to the list of Interaction_Types that have
//  this Molecule_Type as an Input

{
    Vector__Int_Types_For_Input.push_back(Int_Type__Ptr);
}
//----------------------------------------------------------------------------------------------------------

void    Molecule_Type::List_Of_Int_Types_With_MolType_As_Input_Class::Clear()

//   Clear the Molecule Type's List of Interaction Types with the Mol type as an Input

{
    Vector__Int_Types_For_Input.clear();
}
//----------------------------------------------------------------------------------------------------------------
Interaction_Type*  Molecule_Type::List_Of_Int_Types_With_MolType_As_Input_Class::Get_Current_Item()

//  Return the current item in the list 

{
    return *Iter_Int_Type_With_Mol_Type_As_Input;
}
//----------------------------------------------------------------------------------------------------------

bool    Molecule_Type::List_Of_Int_Types_With_MolType_As_Input_Class::Iter_To_First()

//  Move the list Iterator to the first item in the list

{
    if (Vector__Int_Types_For_Input.empty() == true)
    THEN
        return false;
    ELSE
        Iter_Int_Type_With_Mol_Type_As_Input = Vector__Int_Types_For_Input.begin();
        return true;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
bool     Molecule_Type::List_Of_Int_Types_With_MolType_As_Input_Class::Iter_To_Next()

//  Move the List Iterator to the next item in the list

{

    Assert(Iter_Int_Type_With_Mol_Type_As_Input != Vector__Int_Types_For_Input.end(),

           "Molecule_Type::List_Of_Int_Types_With_MolType_As_Input_Class::Iter_To_Next - Attempt to iterate beyond end");

    Iter_Int_Type_With_Mol_Type_As_Input++;

    if (Iter_Int_Type_With_Mol_Type_As_Input == Vector__Int_Types_For_Input.end())
    THEN
        return false;
    ELSE
        return true;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void    Molecule_Type::List_Of_Int_Types_With_MolType_As_Input_Class::Remove(Interaction_Type* Int_Type__Ptr)

//  Remove the specified Interaction_Type from the Molecule_type's list of inputs
//  This function is not optimised, and so is suitable only for use when infrequently called

{
    vector<Interaction_Type*>::iterator  Iter;
    Iter = find(Vector__Int_Types_For_Input.begin(), Vector__Int_Types_For_Input.end(), Int_Type__Ptr);

    Assert (Iter != Vector__Int_Types_For_Input.end(),
            "Molecule_Type::List_Of_Int_Types_With_MolType_As_Input_Class::Remove - Invalid argument");

    Vector__Int_Types_For_Input.erase(Iter);
}
//----------------------------------------------------------------------------------------------------------
