/****************************************************************************
*
*	  Header Filename:  Form_Simulation_Statistics.h
*	  Module Name:      Form_Simulation_Statistics
*	  Description:      Manages user interface for display of Simulation Statistics          
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:- 
*
*          Version      Date                        Update Description
*          =======      ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Form_Simulation_Statistics_H
#define Form_Simulation_Statistics_H

//--------------  GUI Presentation Subsystem API ----------------------------------------------------------------
#include  "GUI_Presentation_API.h"
//---------------------------------------------------------------------------------------------------------------

class   Simulation_Statistics;
class   Indexed_Series;

class   Form_Simulation_Statistics
{
//===== Relationships with other classes ===================================================================
    private:
        Simulation_Statistics*   SimStats_Ptr;
//==========================================================================================================

    public:
        void        Clear();
        enum        Display_Option {Full, Optimised};
        void        Display_Statistics(Display_Option Option, bool Simulation_Running);
        static      Form_Simulation_Statistics* Get_Ptr(); 
        enum        Display_Group_Id {Manhattan_Plots, Series_Plots, None};
        int         Get_Last_Display_Time() {return Last_Display_Time;};
        double      Get_Manhattan_Plot_Sensitivity();
        void        Set_Manhattan_Plot_Sensitivity(double Sensitivity);
        bool        Is_On_Active_Page();
        void        List_Groups();
        void        List_Series_Lists();
        void        List_Series();
        void        ListView_Series_DblClick();
        void        ListView_Group_DblClick();
        void        ListView_SerList_Select();
        void        Page_Select();

        static void Make();
        void        Save_Display_To_File(string* Filename_Ptr);
        string      Toggle_Series_Display(Indexed_Series* Indexed_Series_Ptr);

    private:
        struct  Screen_Plot_Area
        {
            int Left;
            int Right;
            int Top;
            int Bottom;
        };
        Screen_Plot_Area SimStats_Plot_Area;
        
        struct    Screen_Loc    //      Used to define a Screen Location in Pixels
        {
            int X;
            int Y;
        };
        
        struct   X_Y_Coords      
        {
            double X;
            double Y;
        };

        int Manhattan_Plot_Last_Index_Min;
        int Manhattan_Plot_Last_Index_Max;
        double Manhattan_Plot_Sensitivity;
        int    Last_Display_Time;
        bool   SimStats_Has_Been_Displayed;

        static bool Compare_Indexed_Series_Ranges(pair <Indexed_Series*, double> x, pair <Indexed_Series*, double> y);
        void    Display_Series_Name(Indexed_Series*);
        void    Display_Series_Plots(Display_Option Option);
        void    Display_Manhattan_Plot(Display_Option Display_Opt, bool Simulation_Running); 
        void    Draw_Series_X_Axis(Indexed_Series* Series_Ptr, Colour_Id Display_Colour);
        void    Draw_X_Axis(double First_Tick, double Last_Tick, double Increment, Colour_Id Display_Colour, string Axis_Name); 
        void    Draw_Series_Y_Axis(Indexed_Series* Series_Ptr, Colour_Id Display_Colour, string Axis_Name, int Axis_Number);
        void    Draw_Y_Axis(double First_Tick, double Last_Tick, double Increment, Colour_Id Display_Colour, 
                            string Axis_Name, int Axis_Number);
        Form_Simulation_Statistics();

        void    Init_Series_Display();
        void    Plot_Indexed_Series(Indexed_Series* Series_Ptr, 
                                    Colour_Id Display_Colour, 
                                    bool Replot_Flag,
                                    int Max_Index);
        list<Colour_Id>  Available_Plot_Colours;
        struct Series_And_Colour_To_Plot {Indexed_Series* Series_Ptr; Colour_Id Plot_Colour;};
        list <Series_And_Colour_To_Plot> List_Of_Series_To_Plot;
        struct  Display_Group_Spec 
            {
                public:
                    Display_Group_Id  Group_Id; 
                    string Group_Name; 
            };
        void    Get_ListView_Series_Selected_Row(list<string>& List_Of_Col_Texts, 
                                                 Indexed_Series*& Indexed_Series_Ptr);
        void    Get_ListView_Groups_Selected_Row(list<string>& List_Of_Col_Texts, 
                                                 Display_Group_Spec*& Disp_Group_Spec_Ptr);

        vector<Display_Group_Spec* >  List_Of_Display_Group_Specs;
        Display_Group_Id  Current_Display_Group;
        string  Curr_SerList_Name_Tx_;  // Current Series List
        
        static  Form_Simulation_Statistics* Instance_Ptr;
        Form_Simulation_Statistics::Screen_Loc   Map_X_Y_Coords_To_Screen_Loc
                                                   (const X_Y_Coords& Low_Vals, 
                                                    const X_Y_Coords& High_Vals, 
                                                    const X_Y_Coords& Dat_Pair);
                                                                             
        int     Map_X_Val_To_Screen_X(const double& X_Low, const double& X_High, const double& X_Val);
        int     Map_Y_Val_To_Screen_Y(const double& Y_Low, const double& Y_High, const double& Y_Val);
        unsigned int     Num_Series_For_Display();
        void    Join_Screen_Locs(const Screen_Loc Start_Screen_Loc,
                                 const Screen_Loc End_Screen_Loc,
                                 const int  Line_Width,
                                 const Colour_Id Line_Colour);
};
#endif  //Form_Simulation_Statistics_H

