/****************************************************************************
*
*     Module Filename:  Form_System_Monitor.cpp
*     Module Name:      System Monitor Form
*     Description:      System Monitor User Interface Manager                
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*     Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include    "globldef.h"
#include    "Library_Headers.h"

//--------------  GUI Presentation Subsystem  --------------------------------------------------------------------
#include  "GUI_Presentation_API.h"
//-----------------------------------------------------------------------------------------------------------------

//-------------- User Interface Manager -------------------------------------------------------------------------
#include    "Form_System_Monitor.h"
#include    "Form_Main.h"
//---------------------------------------------------------------------------------------------------------------

Form_System_Monitor* Form_System_Monitor::Instance_Ptr = 0;
//==============================================================================================================

void  Form_System_Monitor::Display_Object_Statistics()

//  Display statistics of objects, object creations and object deletions for each class
//  as recorded by the System Monitor. Also record the totals.

{
    int Total_Objects = 0;
    int Total_Creates = 0;
    int Total_Deletes = 0;
    System_Monitor*  SysMon_Ptr = System_Monitor::Get_Ptr();;
    GUI_Presentation_API::Clear(Widget_Ids::SysMon_ListStore);

    for(bool Continue_Flag = SysMon_Ptr->Iter_To_First_Class();
        Continue_Flag == true;
        Continue_Flag = SysMon_Ptr->Iter_To_Next_Class())
    BEGIN
        list<string>  List_Of_Strings;

        int Objects = SysMon_Ptr->Get_Num_Objects_Current_Class();
        int Creates = SysMon_Ptr->Get_Num_Creates_Current_Class();
        int Deletes = SysMon_Ptr->Get_Num_Deletes_Current_Class();

        Total_Objects += Objects;
        Total_Creates += Creates;
        Total_Deletes += Deletes;

        List_Of_Strings.push_back(SysMon_Ptr->Get_Current_Class());
        List_Of_Strings.push_back(To_String(Objects));
        List_Of_Strings.push_back(To_String(Creates));
        List_Of_Strings.push_back(To_String(Deletes));

        GUI_Presentation_API::Add_Row(Widget_Ids::SysMon_ListStore, List_Of_Strings, 0); 
    END     

    list<string> Total_Strings;
    Total_Strings.push_back(string(To_String("TOTAL")));
    Total_Strings.push_back(string(To_String(Total_Objects)));
    Total_Strings.push_back(string(To_String(Total_Creates)));
    Total_Strings.push_back(string(To_String(Total_Deletes)));

    GUI_Presentation_API::Add_Row(Widget_Ids::SysMon_ListStore, Total_Strings, 0); 
}
//-------------------------------------------------------------------------------------------------------------- 

Form_System_Monitor*  Form_System_Monitor::Get_Ptr()

//  Get the pointer to the singleton Form_System_Monitor object.
//  Its a fatal error if the object has not yet been created.

{
    Assert(Instance_Ptr not_eq 0,"Form_System_Monitor::Get_Ptr: Called before an instance has been created");
    return (Instance_Ptr);
}
//--------------------------------------------------------------------------------------------------------------
 
bool   Form_System_Monitor::Is_On_Active_Page()

//  Return true if the System Monitor Tab 
//  is on the currently active page

{   
    string Name = GUI_Presentation_API::Get_Active_Page_Label();
    if (Name == string(""))
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//--------------------------------------------------------------------------------------------------------------

void Form_System_Monitor::Make()

//  Initialise the Form_System_Monitor object 
//  Its a fatal error if the object already exists

{
    Assert(Instance_Ptr == 0,"Form_System_Monitor::Make -  Invalid call; Form_System_Monitor has already been created");
    CREATE_OBJECT(Form_System_Monitor, Instance_Ptr = new Form_System_Monitor);
}
