/****************************************************************************
*
*     Header Filename:  Colour.cpp
*     Module Name:      Colour
*     Description:      Colour Settings for use with GTK
*
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include    "globldef.h"
#include    "Library_Headers.h"

#include <gtkmm.h>

//--------------  GUI Presentation Subsystem ---------------------------------------------------------------------

#include  "Colour.h"
//-----------------------------------------------------------------------------------------------------------------

Colour*  Colour::Instance_Ptr = 0;

//-----------------------------------------------------------------------------------------------------------------
Colour::Colour()

//  Constructor - Setup colours and Colour Mapping

{  
    Assert (Instance_Ptr == 0,"Colour::Colour() - This singleton object has already been constructed");
    Instance_Ptr = this;  // For singleton object

    ColourMap = Gdk::Colormap::get_system();

    CREATE_OBJECT(Gdk::Color, Blue_Ptr = new Gdk::Color());
    Blue_Ptr->set_rgb(0,0,0xffff);

    CREATE_OBJECT(Gdk::Color, Red_Ptr = new Gdk::Color());
    Red_Ptr->set_rgb(0xffff,0,0);

    CREATE_OBJECT(Gdk::Color, Green_Ptr = new Gdk::Color());
    Green_Ptr->set_rgb(0,0x8888,0);

    CREATE_OBJECT(Gdk::Color, Black_Ptr = new Gdk::Color());
    Black_Ptr->set_rgb(0,0,0);

    CREATE_OBJECT(Gdk::Color, White_Ptr = new Gdk::Color());
    White_Ptr->set_rgb(0xffff,0xffff,0xffff);

    CREATE_OBJECT(Gdk::Color, LimeGreen_Ptr = new Gdk::Color);
    LimeGreen_Ptr->set_rgb(0x31ce,0xcc33,0x31ce);

    CREATE_OBJECT(Gdk::Color, Maroon_Ptr = new Gdk::Color);
    Maroon_Ptr->set_rgb(0xaf50,0x2fd0,0x5fa0);
    
    ColourMap->alloc_color(*Blue_Ptr);
    ColourMap->alloc_color(*Red_Ptr);
    ColourMap->alloc_color(*Green_Ptr);   
    ColourMap->alloc_color(*Black_Ptr);
    ColourMap->alloc_color(*White_Ptr);
    ColourMap->alloc_color(*LimeGreen_Ptr);
    ColourMap->alloc_color(*Maroon_Ptr);
}
//----------------------------------------------------------------------------------------------------------------

Gdk::Color  Colour::Get_GDK_Colour_RGB(int Red, int Green, int Blue)

//  Get a Gdk::Color object for the specified RGB parameters

{
    Assert (Instance_Ptr not_eq 0,"Colour::Get_GDK_Colour_RGB() - This singleton object has not been constructed");
    Gdk::Color  Colour;
    Colour.set_rgb(Red, Green, Blue);
    Instance_Ptr->ColourMap->alloc_color(Colour);
    return Colour;
}


