/****************************************************************************
*
*	  Header Filename:  Random_Sequence.h
*	  Module Name:      Random_Sequence
*	  Description:      Maintains sequences of random numbers and can provide 
*                       random numbers based on the underlying sequences
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version	  Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Random_Sequence_H
#define Random_Sequence_H

class   Random_Sequence

{
    public:
        static bool    Check_Portability();
        static void Clear_All();
        void Set_Seed(unsigned int Seed);
        int Uniform(int Maximum);
        double Uniform(double Maximum);
        static void Make(int Num_Sequences_To_Make);
        enum Rand_Sequence_Number {One, Two, Three, Num_Sequences = 2};

        static Random_Sequence*  Get_Ptr(Rand_Sequence_Number Seq_Num)
        //  Return a pointer to the requested Random Sequence instance
            {
                Assert(Seq_Num < Number_Of_Sequences, "Random_Sequence::Get_Ptr - Attempt to access Sequence that does not exist");
                return (*List_Of_Instance_Ptrs_Ptr)[Seq_Num];
            }

 
    private:  

        Random_Sequence(unsigned long int Seed);
        int Current_Index;
        int Cache_Size;
        vector<int>  Sequence;
        static vector<Random_Sequence*>*  List_Of_Instance_Ptrs_Ptr;
        static int Number_Of_Sequences;
        const gsl_rng_type * T;  // Which holds static information about each type of generator
        gsl_rng * r;             // Which describes an instance of a generator created from a given gsl_rng_type
};
typedef  Random_Sequence  RndSeq;

#endif  //Random_Sequence_H
